@@ -1,4 +1,7 @@
-/* Conky, a system monitor, based on torsmo
+/* -*- mode: c; c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*-
+ * vim: ts=4 sw=4 noet ai cindent syntax=c
+ *
+ * Conky, a system monitor, based on torsmo
  *
  * Any original torsmo code is licensed under the BSD license
  *
@@ -7,7 +10,7 @@
  * Please see COPYING for details
  *
  * Copyright (c) 2004, Hannu Saransaari and Lauri Hakkarainen
- * Copyright (c) 2005-2009 Brenden Matthews, Philip Kovacs, et. al.
+ * Copyright (c) 2005-2010 Brenden Matthews, Philip Kovacs, et. al.
  *	(see AUTHORS)
  * All rights reserved.
  *
@@ -23,8 +26,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
- * vim: ts=4 sw=4 noet ai cindent syntax=c
- *
  */
 
 #include "config.h"
@@ -43,13 +44,19 @@
 
 #ifdef XFT
 #include <X11/Xft/Xft.h>
-int use_xft = 0;
+int use_xft;
 #endif
 
 #ifdef HAVE_XDBE
 int use_xdbe;
 #endif
 
+#ifdef USE_ARGB
+int use_argb_visual;
+int have_argb_visual;
+#endif /* USE_ARGB */
+int own_window_argb_value;
+
 /* some basic X11 stuff */
 Display *display = NULL;
 int display_width;
@@ -63,6 +70,7 @@ int workarea[4];
 
 /* Window stuff */
 struct conky_window window;
+char window_created = 0;
 
 /* local prototypes */
 static void update_workarea(void);
@@ -119,18 +127,14 @@ static Window find_desktop_window(Window *p_root, Window *p_desktop)
 
 	XQueryTree(display, root, &troot, &parent, &children, &n);
 	for (i = 0; i < (int) n; i++) {
-		if ((XGetWindowProperty(display, children[i], ATOM(__SWM_VROOT), 0,
-						1, False, XA_WINDOW, &type, &format,
-						&nitems, &bytes, &buf) == Success ||
-					XGetWindowProperty(display, children[i],
-						ATOM(_NET_CURRENT_DESKTOP), 0, 1, False, XA_WINDOW, &type,
-						&format, &nitems, &bytes, &buf) == Success) &&
-				type ==	XA_WINDOW) {
+		if (XGetWindowProperty(display, children[i], ATOM(__SWM_VROOT), 0, 1,
+					False, XA_WINDOW, &type, &format, &nitems, &bytes, &buf)
+				== Success && type == XA_WINDOW) {
 			win = *(Window *) buf;
 			XFree(buf);
 			XFree(children);
 			fprintf(stderr,
-					PACKAGE_NAME": desktop window (%lx) found from __SWM_VROOT or _NET_CURRENT_DESKTOP property\n",
+					PACKAGE_NAME": desktop window (%lx) found from __SWM_VROOT property\n",
 					win);
 			fflush(stderr);
 			*p_root = win;
@@ -173,63 +177,90 @@ static Window find_desktop_window(Window *p_root, Window *p_desktop)
 	return win;
 }
 
-/* sets background to ParentRelative for the Window and all parents */
-void set_transparent_background(void)
+static int colour_set = -1;
+/* if no argb visual is configured sets background to ParentRelative for the Window and all parents,
+   else real transparency is used */
+void set_transparent_background(Window win, int alpha)
 {
-	static int colour_set = -1;
+	(void)alpha; /* disable warnings when unused */
+
+#ifdef USE_ARGB
+	if (have_argb_visual) {
+		// real transparency
+		if (set_transparent) {
+			XSetWindowBackground(display, win, 0x00);
+		} else if (colour_set != background_colour) {
+			XSetWindowBackground(display, win,
+				background_colour | (alpha << 24));
+			colour_set = background_colour;
+		}
+	} else {
+#endif /* USE_ARGB */
+	// pseudo transparency
 
 	if (set_transparent) {
-		int format, i;
-		Atom type;
-		unsigned long nitems, bytes;
-		unsigned int n;
-		Window root = RootWindow(display, screen);
-		Pixmap *pixmap = 0;
-		if (
-				(
-				 XGetWindowProperty(display, root, ATOM(_XROOTPMAP_ID), 0,
-					 1, False, ATOM(PIXMAP), &type, &format,
-					 &nitems, &bytes, (unsigned char**)&pixmap) == Success ||
-				 XGetWindowProperty(display, root, ATOM(ESETROOT_PMAP_ID), 0,
-					 1, False, ATOM(PIXMAP), &type, &format,
-					 &nitems, &bytes, (unsigned char**)&pixmap) == Success
-				)
-				&&
-				nitems == 1L && format == 32) {
-			GC gc = XCreateGC(display, window.drawable, 0, NULL);
-			XCopyArea(display, *pixmap, window.drawable, gc, window.x, window.y, window.width, window.height, 0, 0);
-			XFreeGC(display, gc);
-		} else {
-			Window parent = window.window;
-			for (i = 0; i < 50 && parent != root; i++) {
-				Window r, *children;
+		Window parent = win;
+		unsigned int i;
 
-				XSetWindowBackgroundPixmap(display, parent, ParentRelative);
+		for (i = 0; i < 50 && parent != RootWindow(display, screen); i++) {
+			Window r, *children;
+			unsigned int n;
 
-				XQueryTree(display, parent, &r, &parent, &children, &n);
-				XFree(children);
-			}
-		}
-		if (pixmap) {
-			XFree(pixmap);
+			XSetWindowBackgroundPixmap(display, parent, ParentRelative);
+
+			XQueryTree(display, parent, &r, &parent, &children, &n);
+			XFree(children);
 		}
 	} else if (colour_set != background_colour) {
-		XSetWindowBackground(display, window.window, background_colour);
+		XSetWindowBackground(display, win, background_colour);
 		colour_set = background_colour;
 	}
+#ifdef USE_ARGB
+	}
+#endif /* USE_ARGB */
 }
 
+#ifdef USE_ARGB
+static int get_argb_visual(Visual** visual, int *depth) {
+	/* code from gtk project, gdk_screen_get_rgba_visual */
+	XVisualInfo visual_template;
+	XVisualInfo *visual_list;
+	int nxvisuals = 0, i;
+
+	visual_template.screen = screen;
+	visual_list = XGetVisualInfo (display, VisualScreenMask,
+				&visual_template, &nxvisuals);
+	for (i = 0; i < nxvisuals; i++) {
+		if (visual_list[i].depth == 32 &&
+			 (visual_list[i].red_mask   == 0xff0000 &&
+			  visual_list[i].green_mask == 0x00ff00 &&
+			  visual_list[i].blue_mask  == 0x0000ff)) {
+			*visual = visual_list[i].visual;
+			*depth = visual_list[i].depth;
+			DBGP("Found ARGB Visual");
+			XFree(visual_list);
+			return 1;
+		}
+	}
+	// no argb visual available
+	DBGP("No ARGB Visual found");
+	XFree(visual_list);
+	return 0;
+}
+#endif /* USE_ARGB */
+
 void destroy_window(void)
 {
 #ifdef XFT
-	if (window.xftdraw) {
+	if(window.xftdraw) {
 		XftDrawDestroy(window.xftdraw);
 	}
-#endif /* XFT */
-	if (window.gc) {
+#endif
+	if(window.gc) {
 		XFreeGC(display, window.gc);
 	}
 	memset(&window, 0, sizeof(struct conky_window));
+	colour_set = -1;
 }
 
 void init_window(int own_window, int w, int h, int set_trans, int back_colour,
@@ -240,13 +271,33 @@ void init_window(int own_window, int w, int h, int set_trans, int back_colour,
 	 * happens but I bet the bug is somewhere here. */
 	set_transparent = set_trans;
 	background_colour = back_colour;
+	window_created = 1;
 
 #ifdef OWN_WINDOW
 	if (own_window) {
+		int depth = 0, flags;
+		Visual *visual = NULL;
+
 		if (!find_desktop_window(&window.root, &window.desktop)) {
 			return;
 		}
 
+#ifdef USE_ARGB
+		if (use_argb_visual && get_argb_visual(&visual, &depth)) {
+			have_argb_visual = 1;
+			window.visual = visual;
+			window.colourmap = XCreateColormap(display,
+				DefaultRootWindow(display), window.visual, AllocNone);
+		} else {
+#endif /* USE_ARGB */
+			window.visual = DefaultVisual(display, screen);
+			window.colourmap = DefaultColormap(display, screen);
+			depth = CopyFromParent;
+			visual = CopyFromParent;
+#ifdef USE_ARGB
+		}
+#endif /* USE_ARGB */
+
 		if (window.type == TYPE_OVERRIDE) {
 
 			/* An override_redirect True window.
@@ -254,11 +305,21 @@ void init_window(int own_window, int w, int h, int set_trans, int back_colour,
 			XSetWindowAttributes attrs = { ParentRelative, 0L, 0, 0L, 0, 0,
 				Always, 0L, 0L, False, StructureNotifyMask | ExposureMask, 0L,
 				True, 0, 0 };
+#ifdef USE_ARGB
+			if (have_argb_visual) {
+				attrs.colormap = window.colourmap;
+				flags = CWBorderPixel | CWColormap | CWOverrideRedirect;
+			} else {
+#endif /* USE_ARGB */
+				flags = CWBackPixel | CWOverrideRedirect;
+#ifdef USE_ARGB
+			}
+#endif /* USE_ARGB */
 
 			/* Parent is desktop window (which might be a child of root) */
 			window.window = XCreateWindow(display, window.desktop, window.x,
-					window.y, w, h, 0, CopyFromParent, InputOutput, CopyFromParent,
-					CWBackPixel | CWOverrideRedirect, &attrs);
+					window.y, w, h, 0, depth, InputOutput, visual,
+					flags, &attrs);
 
 			XLowerWindow(display, window.window);
 
@@ -276,13 +337,24 @@ void init_window(int own_window, int w, int h, int set_trans, int back_colour,
 			XWMHints wmHint;
 			Atom xa;
 
+#ifdef USE_ARGB
+			if (have_argb_visual) {
+				attrs.colormap = window.colourmap;
+				flags = CWBorderPixel | CWColormap | CWOverrideRedirect;
+			} else {
+#endif /* USE_ARGB */
+				flags = CWBackPixel | CWOverrideRedirect;
+#ifdef USE_ARGB
+			}
+#endif /* USE_ARGB */
+
 			if (window.type == TYPE_DOCK) {
 				window.x = window.y = 0;
 			}
 			/* Parent is root window so WM can take control */
 			window.window = XCreateWindow(display, window.root, window.x,
-					window.y, w, h, 0, CopyFromParent, InputOutput, CopyFromParent,
-					CWBackPixel | CWOverrideRedirect, &attrs);
+					window.y, w, h, 0, depth, InputOutput, visual,
+					flags, &attrs);
 
 			classHint.res_name = window.class_name;
 			classHint.res_class = classHint.res_name;
@@ -297,8 +369,9 @@ void init_window(int own_window, int w, int h, int set_trans, int back_colour,
 				wmHint.initial_state = NormalState;
 			}
 
-			XmbSetWMProperties(display, window.window, window.title, NULL, argv,
+			XmbSetWMProperties(display, window.window, NULL, NULL, argv,
 					argc, NULL, &wmHint, &classHint);
+			XStoreName(display, window.window, window.title);
 
 			/* Sets an empty WM_PROTOCOLS property */
 			XSetWMProtocols(display, window.window, NULL, 0);
@@ -430,13 +503,13 @@ void init_window(int own_window, int w, int h, int set_trans, int back_colour,
 			}
 
 			/* Skip pager */
-			if (TEST_HINT(window.hints, HINT_SKIP_PAGER)) {
+			if (TEST_HINT(window.hints, HINT_FULLSCREEN)) {
 				/* fprintf(stderr, PACKAGE_NAME": hint - skip_pager\n");
 				   fflush(stderr); */
 
 				xa = ATOM(_NET_WM_STATE);
 				if (xa != None) {
-					Atom xa_prop = ATOM(_NET_WM_STATE_SKIP_PAGER);
+					Atom xa_prop = ATOM(_NET_WM_STATE_FULLSCREEN);
 
 					XChangeProperty(display, window.window, xa, XA_ATOM, 32,
 							PropModeAppend, (unsigned char *) &xa_prop, 1);
@@ -458,6 +531,8 @@ void init_window(int own_window, int w, int h, int set_trans, int back_colour,
 		if (!window.window) {
 			window.window = find_desktop_window(&window.root, &window.desktop);
 		}
+		window.visual = DefaultVisual(display, screen);
+		window.colourmap = DefaultColormap(display, screen);
 
 		if (XGetWindowAttributes(display, window.window, &attrs)) {
 			window.width = attrs.width;
@@ -487,15 +562,13 @@ void init_window(int own_window, int w, int h, int set_trans, int back_colour,
 			}
 		}
 		if (!use_xdbe) {
-			ERR("failed to set up double buffer");
+			NORM_ERR("failed to set up double buffer");
 		}
 	}
 	if (!use_xdbe) {
 		fprintf(stderr, PACKAGE_NAME": drawing to single buffer\n");
 	}
 #endif
-	window.visual = DefaultVisual(display, DefaultScreen(display));
-	window.colourmap = DefaultColormap(display, DefaultScreen(display));
 #ifdef IMLIB2
 	{
 		cimlib_init(display, window.drawable, window.visual, window.colourmap);
@@ -546,32 +619,6 @@ static Window find_subwindow(Window win, int w, int h)
 	return win;
 }
 
-long get_x11_color(const char *name)
-{
-	XColor color;
-
-	color.pixel = 0;
-	if (!XParseColor(display, DefaultColormap(display, screen), name, &color)) {
-		/* lets check if it's a hex colour with the # missing in front
-		 * if yes, then do something about it */
-		char newname[DEFAULT_TEXT_BUFFER_SIZE];
-
-		newname[0] = '#';
-		strncpy(&newname[1], name, DEFAULT_TEXT_BUFFER_SIZE - 1);
-		/* now lets try again */
-		if (!XParseColor(display, DefaultColormap(display, screen), &newname[0],
-					&color)) {
-			ERR("can't parse X color '%s'", name);
-			return 0xFF00FF;
-		}
-	}
-	if (!XAllocColor(display, DefaultColormap(display, screen), &color)) {
-		ERR("can't allocate X color '%s'", name);
-	}
-
-	return (long) color.pixel;
-}
-
 void create_gc(void)
 {
 	XGCValues values;
@@ -728,11 +775,14 @@ void get_x11_desktop_info(Display *current_display, Atom atom)
 	}
 }
 
-void update_x11info(void)
+int update_x11info(void)
 {
 	struct information *current_info = &info;
+	if (x_initialised != YES)
+		return 0;
 	current_info->x11.monitor.number = XScreenCount(display);
 	current_info->x11.monitor.current = XDefaultScreen(display);
+	return 0;
 }
 
 #ifdef OWN_WINDOW
@@ -801,3 +851,17 @@ void set_struts(int sidenum)
 	}
 }
 #endif /* OWN_WINDOW */
+
+#ifdef HAVE_XDBE
+void xdbe_swap_buffers(void)
+{
+	if (use_xdbe) {
+		XdbeSwapInfo swap;
+
+		swap.swap_window = window.window;
+		swap.swap_action = XdbeBackground;
+		XdbeSwapBuffers(display, &swap, 1);
+	}
+}
+#endif /* HAVE_XDBE */
+
