@@ -1,3 +1,5 @@
+/* -*- mode: c; c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*- */
+
 #ifdef X11
 #ifndef X11_H_
 #define X11_H_
@@ -31,7 +33,7 @@ enum _window_hints {
 	HINT_ABOVE,
 	HINT_STICKY,
 	HINT_SKIP_TASKBAR,
-	HINT_SKIP_PAGER
+	HINT_FULLSCREEN
 };
 
 #define SET_HINT(mask, hint)	(mask |= (1 << hint))
@@ -73,6 +75,15 @@ extern int use_xdbe;
 extern int use_xft;
 #endif
 
+#if defined(USE_ARGB) && defined(OWN_WINDOW)
+/* 1 if config var set to 1, otherwise 0 */
+extern int use_argb_visual;
+/* 1 if use_argb_visual=1 and argb visual was found, otherwise 0 */
+extern int have_argb_visual;
+#endif
+/* range of 0-255 for alpha */
+extern int own_window_argb_value;
+
 extern Display *display;
 extern int display_width;
 extern int display_height;
@@ -81,16 +92,20 @@ extern int screen;
 extern int workarea[4];
 
 extern struct conky_window window;
+extern char window_created;
 
 void init_X11(const char*);
 void init_window(int use_own_window, int width, int height, int set_trans,
 	int back_colour, char **argv, int argc);
 void destroy_window(void);
 void create_gc(void);
-void set_transparent_background(void);
-long get_x11_color(const char *);
+void set_transparent_background(Window win, int alpha);
 void get_x11_desktop_info(Display *display, Atom atom);
 void set_struts(int);
 
+#ifdef HAVE_XDBE
+void xdbe_swap_buffers(void);
+#endif /* HAVE_XDBE */
+
 #endif /*X11_H_*/
 #endif /* X11 */
